package com.muzhi.donghang.controller;

import com.muzhi.donghang.common.R;
import com.muzhi.donghang.service.EmployeeService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;


@Api(description = "钉钉人事")
@RestController
@RequestMapping("/empservice")
@CrossOrigin//解决跨域问题
public class EmployeeController {

    @Autowired
    private EmployeeService employeeService;

    //获取钉钉员工花名册主部门ID
    @GetMapping("getMainDeptId")
    public String getMainDeptId(String userid){
        //TODO
        //userid = "manager5889";//牧之测试架构，牧之userid
//        userid = "0500382234808088";//东航架构，才浅userid

        String employee = employeeService.getMainDeptId(userid);

        return employee;
    }

    //获取指定部门的所有父部门id列表
    @GetMapping("getDeptIdList")
    public List getDeptIdList(String userid){
        //TODO
        //userid = "manager5889";//牧之测试架构，牧之userid
//        userid = "0500382234808088";//东航架构，才浅userid

        List deptIdList = employeeService.getDeptIdList(userid);

        return deptIdList;
    }

    //======【最终接口】获取主架构部门名称======
    @GetMapping("getMainDeptName")
    public R getMainDeptName(){

        String mainDeptName = employeeService.getMainDeptName();

        if (mainDeptName == null || mainDeptName == ""){
            return R.error().message("获取主架构部门失败！");
        }else {
            return R.ok().data("mainDeptName",mainDeptName);
        }
    }

    //======【最终接口】根据所有的父部门id列表获取各个部门的名称======
    @GetMapping("getDeptName")
    public R getDeptName(String userid){
        //TODO
        //userid = "manager5889";//牧之测试架构，牧之userid
//        userid = "0500382234808088";//东航架构，才浅userid

        String deptName = employeeService.getDeptName(userid);

        if (deptName == null || deptName == ""){
            return R.error().message("获取部门层级失败！");
        }else {
            return R.ok().data("deptName",deptName);
        }
    }

    //======【最终接口】获取钉钉员工入职日期，并换算入职天数======
    @GetMapping("getJoinDay")
    public R getJoinDay(String userid){
        //TODO
        //userid = "manager5889";//牧之测试架构，牧之userid
//        userid = "0500382234808088";//东航架构，才浅userid

        String joinDay = employeeService.getJoinDay(userid);

        if (joinDay == null || joinDay == ""){
            return R.error().message("获取员工入职天数失败，请检查钉钉中该人员入职日期是否填写！");
        }else {
            return R.ok().data("joinDay",joinDay);
        }
    }

    //======【最终接口】当前人员的本月实际出勤天数======
    @GetMapping("getChuQinDays")
    public R getChuQinDays(String userid){
        //TODO
        //userid = "manager5889";//牧之测试架构，牧之userid
//        userid = "0500382234808088";//东航架构，才浅userid

        String chuQinDays = employeeService.getChuQinDays(userid);

        if (chuQinDays == null || chuQinDays == ""){
            return R.error().message("获取员工本月出勤天数失败！");
        }else {
            return R.ok().data("chuQinDays",chuQinDays);
        }
    }

    //===【最终接口】获取当前人员的主部门下的人员，并循环获取每个人员的考勤打卡===
    @GetMapping("getDeptDaKa")
    public R getDeptDaKa(String userid){
        //TODO
        //userid = "manager5889";//牧之测试架构，牧之userid
//        userid = "0500382234808088";//东航架构，才浅userid

        String deptDaKa = employeeService.getDeptDaKa(userid);

        if (deptDaKa == null || deptDaKa == ""){
            return R.error().message("获取当前部门出勤人数失败！");
        }else {
            return R.ok().data("deptDaKa",deptDaKa);
        }
    }

    //===【最终接口】获取当前人员的主部门下的人员，并循环获取每个人员是否请假===
    @GetMapping("getDeptQingJia")
    public R getDeptQingJia(String userid){
        //TODO
        //userid = "manager5889";//牧之测试架构，牧之userid
//        userid = "0500382234808088";//东航架构，才浅userid

        String deptQingJia = employeeService.getDeptQingJia(userid);

        if (deptQingJia == null || deptQingJia == ""){
            return R.error().message("获取当前部门请假人数失败！");
        }else {
            return R.ok().data("deptQingJia",deptQingJia);
        }
    }

    //===【最终接口】获取钉钉日程信息===
    @GetMapping("getSchedule")
    public R getSchedule(String userid){
        //TODO
        //userid = "manager5889";//牧之测试架构，牧之userid
//        userid = "0500382234808088";//东航架构，才浅userid

        Map schedule = employeeService.getSchedule(userid);
        System.out.println("controller返回：" + schedule);

//        if (schedule == null || schedule.isEmpty()){
//            return R.error().message("获取当前人员日程失败！");
//        }else {
            return R.ok().data(schedule);
//        }
    }

    //===【最终接口】获取人天===
    @GetMapping("getWeather")
    public R getWeather(){

        Map weather = employeeService.getWeather();

//        return R.ok().data("getWeather",weather);
        return R.ok().data(weather);
    }
}